/**
 * Utility for reading a sequence of little endian numbers
 * (c) 2012 Peter Lekensteyn <lekensteyn@gmail.com>
 * License: GPLv3+
 */
#include <stdio.h>
#include <unistd.h>
#include <endian.h>
#include <stdint.h>
#include <errno.h>
#include <stdlib.h>

uint32_t rethex(uint32_t a) {
	return htobe32(a);
}

uint32_t retle(uint32_t a) {
	return htole32(a);
}

int main(int argc, char **argv) {
	FILE *fp;
	unsigned long offset, step, last, i;
	if (argc != 5) {
		fprintf(stderr, "Usage: %s file offset step last\n", *argv);
		return 1;
	}
	offset = atol(argv[2]);
	step = atol(argv[3]);
	last = atol(argv[4]);
	if (step < 0) {
		fprintf(stderr, "step must be larger than zero\n");
		return 1;
	}
	fp = fopen(argv[1], "r");
	if (!fp) {
		perror("fopen");
		return 1;
	}
	uint32_t bytes, (*conv)(uint32_t);
	conv = getenv("LE") ? &retle : &rethex;
	for (i=offset; i<last; i+=step) {
		if (fseek(fp, i, SEEK_SET)) {
			perror("fseek");
			break;
		}
		int rb = fread(&bytes, 4, 1, fp);
		if (rb != 1) {
			perror("fread");
			return 1;
		}
		printf("%8lx %08X\n", i, conv(bytes));
	}
	fclose(fp);
	return 0;
}
